﻿// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// http://www.shawnduggan.com/?p=54
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
using System;
using System.ComponentModel;
using System.IO;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;

namespace SoftArcs.Community.ShawnDuggan.Windows.Controls
{
	/// <summary>
	/// Represents a bindable rich editing control which operates on System.Windows.Documents.FlowDocument objects. 
	/// AFA: The binding target has been changed to use a DependencyPropery "Text". The document property will still be operable
	/// </summary>
	public class ExtendedRichTextBox : RichTextBox
	{
		public string Text
		{
			get { return (string)GetValue( TextProperty ); }
			set { SetValue( TextProperty, value ); }
		}
		public static readonly DependencyProperty TextProperty =
			DependencyProperty.Register( "Text", typeof( string ), typeof( ExtendedRichTextBox ),
												  new PropertyMetadata( default( string ) ) );

		/// <summary>
		/// Initializes a new instance of the <see cref="ExtendedRichTextBox"/> class.
		/// </summary>
		public ExtendedRichTextBox() { }

		/// <summary>
		/// Initializes a new instance of the <see cref="ExtendedRichTextBox"/> class.
		/// </summary>
		/// <param name="document">A <see cref="T:System.Windows.Documents.FlowDocument"></see> to be added as the initial contents of the new <see cref="ExtendedRichTextBox"></see>.</param>
		public ExtendedRichTextBox(FlowDocument document) : base( document ) { }

		/// <summary>
		/// Raises the <see cref="E:System.Windows.FrameworkElement.Initialized"></see> event. This method is invoked whenever <see cref="P:System.Windows.FrameworkElement.IsInitialized"></see> is set to true internally.
		/// </summary>
		/// <param name="e">The <see cref="T:System.Windows.RoutedEventArgs"></see> that contains the event data.</param>
		protected override void OnInitialized(EventArgs e)
		{
			base.OnInitialized( e );

			// Hook up to get notified when TextProperty changes.
			var descriptor = DependencyPropertyDescriptor.FromProperty( TextProperty, typeof( TextBox ) );
			descriptor.AddValueChanged( this, delegate
														 {
															 if (RTBContent != Text) RTBContent = Text ?? string.Empty;
														 } );
			TextChanged += delegate
								{
									if (RTBContent != null && Text != RTBContent) Text = RTBContent;
								};
		}

		#region Text Support Routines

		public string RTBContent
		{
			get
			{
				var textRange = new TextRange( Document.ContentStart, Document.ContentEnd );
				using (var ms = new MemoryStream())
				{
					textRange.Save( ms, DataFormats.Text );
					return Encoding.Default.GetString( ms.ToArray() );
				}
			}
			set
			{
				var selection = new TextRange( Document.ContentStart, Document.ContentEnd );
				SetRangeContent( selection, value );

				if ((string)GetValue( TextBox.TextProperty ) != value)
					SetValue( TextBox.TextProperty, value );
			}
		}

		private string SetRangeContent(TextRange selection, string content)
		{
			var loadType = DataFormats.Rtf;
			content = content == "" ? " " : content;
			using (MemoryStream ms = new MemoryStream( Encoding.Default.GetBytes( content ) ))
			{
				if (content.StartsWith( "<Section" ))
				{
					if (selection.CanLoad( DataFormats.Xaml ))
					{
						selection.Load( ms, DataFormats.Xaml );
						loadType = DataFormats.Xaml;
					}
				}
				else if (content.StartsWith( "PK" ))
				{
					if (selection.CanLoad( DataFormats.XamlPackage ))
					{
						selection.Load( ms, DataFormats.XamlPackage );
						loadType = DataFormats.XamlPackage;
					}
				}
				else if (content.StartsWith( "{\\rtf" ))
				{
					if (selection.CanLoad( DataFormats.Rtf ))
					{
						selection.Load( ms, DataFormats.Rtf );
						loadType = DataFormats.Rtf;
					}
				}
				else
				{
					if (selection.CanLoad( DataFormats.Text ))
					{
						selection.Load( ms, DataFormats.Text );
						loadType = DataFormats.Text;
					}
				}
			}

			return loadType;
		}

		#endregion Text Support Routines
	}
}