﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using SoftArcs.Community.Sheva.Windows.Documents;

namespace SoftArcs.UIClassExtensions
{
	public static class RichTextBoxExtensions
	{
		/// <summary>
		/// Get the content of the FlowDocument as text.
		/// </summary>
		/// <param name="richTextBox"></param>
		/// <returns></returns>
		public static string GetText(this RichTextBox richTextBox)
		{
			var textRange = new TextRange( richTextBox.Document.ContentStart, richTextBox.Document.ContentEnd );
			return textRange.Text;
		}

		/// <summary>
		/// Set the content of the FlowDocument to a specific string (without formatting).
		/// </summary>
		/// <param name="richTextBox"></param>
		/// <param name="text"></param>
		public static void SetText(this RichTextBox richTextBox, string text)
		{
			richTextBox.Document.Blocks.Clear();
			richTextBox.Document.Blocks.Add( new Paragraph( new Run( text ) ) );
		}

		/// <summary>
		/// Replace one text in the FlowDocument with another text.
		/// </summary>
		/// <param name="richTextBox"></param>
		/// <param name="textToFind"></param>
		/// <param name="replacementText"></param>
		public static void ReplaceText(this RichTextBox richTextBox, string textToFind, string replacementText)
		{
			var findAndReplaceManager = new FindAndReplaceManager( richTextBox.Document );
			findAndReplaceManager.Replace( textToFind, replacementText );
		}

		/// <summary>
		/// Create a hyperlink for the given text and add an EventHandler for the click event if desired.
		/// </summary>
		/// <param name="richTextBox"></param>
		/// <param name="hyperlinkText"></param>
		/// <param name="hyperlinkClickedEventHandler"></param>
		/// <param name="newHyperlinkText"></param>
		public static void CreateHyperlink(this RichTextBox richTextBox, string hyperlinkText,
													  RoutedEventHandler hyperlinkClickedEventHandler = null,
													  string newHyperlinkText = "")
		{
			var findAndReplaceManager = new FindAndReplaceManager( richTextBox.Document );

			var currentPosition = richTextBox.Document.ContentStart;
			var textRange = findAndReplaceManager.GetTextRangeFromPosition( ref currentPosition, hyperlinkText );

			if (textRange != null)
			{
				try
				{
					var hyperlink = new Hyperlink( textRange.Start, textRange.End )
					{
						IsEnabled = true,
						NavigateUri = new Uri( hyperlinkText ),
						TargetName = newHyperlinkText
					};

					if (hyperlinkClickedEventHandler != null) hyperlink.Click += hyperlinkClickedEventHandler;
				}
				catch (Exception exception)
				{
					Debug.WriteLine( exception.Message );
				}
			}

			richTextBox.IsDocumentEnabled = true;
		}
	}
}
