using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Interactivity;
using System.Windows.Threading;
using SoftArcs.UIClassExtensions;

namespace SoftArcs.UrlServices.UI.Behaviors
{
	public class UrlShorteningBehavior : Behavior<TextBoxBase>
	{
		#region Constants

		private const double AUTO_SHORTENING_DELAY_TIME = 300.0;

		#endregion

		#region Private Fields

		private BackgroundWorker urlShortener;
		private DispatcherTimer timer;

		#endregion

		#region Public Properties

		/// <summary>
		/// The time span in Milliseconds, after which the auto shortening will start.
		/// </summary>
		public double AutoShorteningDelayTime { get; set; }

		/// <summary>
		/// Flag for the RichTextBox which indicates if hyperlinks are being created or not.
		/// </summary>
		public bool AutoShorten { get; set; }

		#endregion

		#region Method Overrides

		protected override void OnAttached()
		{
			if (AutoShorteningDelayTime.Equals( 0.0 )) AutoShorteningDelayTime = AUTO_SHORTENING_DELAY_TIME;

			AssociatedObject.TextChanged += AssociatedObject_OnTextChanged;

			urlShortener = new BackgroundWorker();
			urlShortener.DoWork += urlShortener_DoWork;
			urlShortener.RunWorkerCompleted += urlShortener_RunWorkerCompleted;

			timer = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds( AutoShorteningDelayTime ) };
			timer.Tick += TimerOnTick;
		}

		#endregion

		#region Event Handler

		private void AssociatedObject_OnTextChanged(object sender, TextChangedEventArgs e)
		{
			if (sender == null || urlShortener.IsBusy) return;

			timer.Stop();
			timer.Start();
		}

		private void TimerOnTick(object sender, EventArgs e)
		{
			timer.Stop();

			if (AssociatedObject is TextBox) urlShortener.RunWorkerAsync( (AssociatedObject as TextBox).Text );
			if (AssociatedObject is RichTextBox) urlShortener.RunWorkerAsync( (AssociatedObject as RichTextBox).GetText() );
		}

		private static void urlShortener_DoWork(object sender, DoWorkEventArgs e)
		{
			e.Result = ShorteningService.ShortenAllOccuringUrls( e.Argument.ToString() );
		}

		private void urlShortener_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			var uriMappings = e.Result as Dictionary<string, string>;

			if (uriMappings == null) return;

			foreach (var uriMapping in uriMappings)
			{
				if (AssociatedObject is TextBox)
				{
					var textBox = AssociatedObject as TextBox;
					textBox.Text = textBox.Text.Replace( uriMapping.Key, uriMapping.Value );
					textBox.SelectionStart = textBox.Text.Length;

					UpdateToolTip( uriMapping.Key, uriMapping.Value );
				}

				if (AssociatedObject is RichTextBox)
				{
					var richTextBox = AssociatedObject as RichTextBox;
					if (AutoShorten)
					{
						richTextBox.ReplaceText( uriMapping.Key, uriMapping.Value );
						UpdateToolTip( uriMapping.Key, uriMapping.Value );
					}
					else
					{
						richTextBox.CreateHyperlink( uriMapping.Key, hyperlink_Click, uriMapping.Value );
					}
				}
			}
		}

		private void hyperlink_Click(object sender, RoutedEventArgs e)
		{
			var richTextBox = AssociatedObject as RichTextBox;
			var hyperlink = sender as Hyperlink;

			if (hyperlink != null)
			{
				richTextBox.ReplaceText( hyperlink.NavigateUri.OriginalString, hyperlink.TargetName );
				UpdateToolTip( hyperlink.NavigateUri.ToString(), hyperlink.TargetName );
			}
		}

		private void UpdateToolTip(string oldUrl, string newUrl)
		{
			if (AssociatedObject.ToolTip != null) AssociatedObject.ToolTip += "\n";
			AssociatedObject.ToolTip += oldUrl + " => " + newUrl;
		}

		#endregion
	}
}
